@@ -327,6 +327,7 @@ except ImportError:
 
 
 # Import Salt Testing libs
+from salttesting import helpers
 from salttesting import version
 from salttesting.unit import TestLoader, TestSuite, TextTestRunner
 from salttesting.xmlunit import HAS_XMLRUNNER, XMLTestRunner
@@ -475,6 +476,7 @@ def print_header(header, sep='~', top=True, bottom=True, inline=False, centered=
 
     if bottom and not inline:
         print(sep * width)
+    sys.stdout.flush()
 
 
 class RootsDict(dict):
@@ -1788,7 +1790,9 @@ class SaltRuntests(argparse.ArgumentParser):
             '  Overall Tests Report  ', sep='=', centered=True, inline=True,
             width=self.options.output_columns
         )
-        return
+
+        # Brute force approach to terminate this process and it's children
+        helpers.terminate_process_pid(os.getpid(), only_children=True)
 
     def finalize(self, exit_code=0):
         '''
