@@ -26,7 +26,9 @@ import warnings
 from functools import partial
 from contextlib import closing
 
+import six
 from salttesting import TestLoader, TextTestRunner
+from salttesting import helpers
 from salttesting.version import __version_info__
 from salttesting.xmlunit import HAS_XMLRUNNER, XMLTestRunner
 try:
@@ -107,7 +109,8 @@ class SaltTestingParser(optparse.OptionParser):
         'salttest/ubuntu-12.04': 'python2.7',
         'salttest/ubuntu-12.10': 'python2.7',
         'salttest/ubuntu-13.04': 'python2.7',
-        'salttest/ubuntu-13.10': 'python2.7'
+        'salttest/ubuntu-13.10': 'python2.7',
+        'salttest/py3': 'python3'
     }
 
     def __init__(self, testsuite_directory, *args, **kwargs):
@@ -185,6 +188,13 @@ class SaltTestingParser(optparse.OptionParser):
             help=('Specific test name to run. A named test is the module path '
                   'relative to the tests directory')
         )
+        self.test_selection_group.add_option(
+            '--names-file',
+            dest='names_file',
+            default=None,
+            help=('The location of a newline delimited file of test names to '
+                  'run')
+        )
         self.add_option_group(self.test_selection_group)
 
         if self.support_docker_execution is True:
@@ -327,6 +337,15 @@ class SaltTestingParser(optparse.OptionParser):
             pass
 
         self.options, self.args = optparse.OptionParser.parse_args(self, args, values)
+        if self.options.names_file:
+            with open(self.options.names_file, 'rb') as fp_:
+                lines = []
+                for line in fp_.readlines():
+                    lines.append(line.strip())
+            if self.options.name:
+                self.options.name.extend(lines)
+            else:
+                self.options.name = lines
 
         print_header(u'', inline=True, width=self.options.output_columns)
         self.pre_execution_cleanup()
@@ -477,20 +496,30 @@ class SaltTestingParser(optparse.OptionParser):
                     shutil.rmtree(path)
 
     def run_suite(self, path, display_name, suffix='[!_]*.py',
-                  load_from_name=False):
+                  load_from_name=False, additional_test_dirs=None):
         '''
         Execute a unit test suite
         '''
+        loaded_custom = False
         loader = TestLoader()
         try:
             if load_from_name:
                 tests = loader.loadTestsFromName(display_name)
             else:
-                tests = loader.discover(path, suffix, self.testsuite_directory)
-        except AttributeError:
+                if additional_test_dirs is None or self.testsuite_directory.startswith(path):
+                    tests = loader.discover(path, suffix, self.testsuite_directory)
+                else:
+                    tests = loader.discover(path, suffix)
+                    loaded_custom = True
+        except (AttributeError, ImportError):
             print('Could not locate test \'{0}\'. Exiting.'.format(display_name))
             sys.exit(1)
 
+        if additional_test_dirs and not loaded_custom:
+            for test_dir in additional_test_dirs:
+                additional_tests = loader.discover(test_dir, suffix, test_dir)
+                tests.addTests(additional_tests)
+
         header = '{0} Tests'.format(display_name)
         print_header('Starting {0}'.format(header),
                      width=self.options.output_columns)
@@ -552,7 +581,7 @@ class SaltTestingParser(optparse.OptionParser):
                 for testcase, reason in results.skipped:
                     print(fmt.format(testcase.id(), reason, maxlen=maxlen))
                 print_header(u' ', sep='-', inline=True,
-                            width=self.options.output_columns)
+                             width=self.options.output_columns)
 
             if results.errors:
                 print_header(
@@ -568,7 +597,7 @@ class SaltTestingParser(optparse.OptionParser):
                     for line in reason.rstrip().splitlines():
                         print('       {0}'.format(line.rstrip()))
                     print_header(u'   ', sep=u'.', inline=True,
-                                width=self.options.output_columns)
+                                 width=self.options.output_columns)
                 print_header(u' ', sep='-', inline=True,
                              width=self.options.output_columns)
 
@@ -586,7 +615,7 @@ class SaltTestingParser(optparse.OptionParser):
                     for line in reason.rstrip().splitlines():
                         print('       {0}'.format(line.rstrip()))
                     print_header(u'   ', sep=u'.', inline=True,
-                                width=self.options.output_columns)
+                                 width=self.options.output_columns)
                 print_header(u' ', sep='-', inline=True,
                              width=self.options.output_columns)
 
@@ -610,7 +639,6 @@ class SaltTestingParser(optparse.OptionParser):
             '  Overall Tests Report  ', sep='=', centered=True, inline=True,
             width=self.options.output_columns
         )
-        return
 
     def post_execution_cleanup(self):
         '''
@@ -626,6 +654,11 @@ class SaltTestingParser(optparse.OptionParser):
         if self.options.no_report is False:
             self.print_overall_testsuite_report()
         self.post_execution_cleanup()
+        # Brute force approach to terminate this process and it's children
+        logging.getLogger(__name__).info('Terminating test suite child processes.')
+        helpers.terminate_process_pid(os.getpid(), only_children=True)
+        logging.getLogger(__name__).info('Terminating test suite child processes if any are still found running.')
+        helpers.terminate_process_pid(os.getpid(), only_children=True)
         logging.getLogger(__name__).info(
             'Test suite execution finalized with exit code: {0}'.format(
                 exit_code
@@ -646,43 +679,50 @@ class SaltTestingParser(optparse.OptionParser):
 
             # Let's check if, in fact, the container is stopped
             scode_call = subprocess.Popen(
-                ['docker', 'inspect', '-format={{.State.Running}}', cid],
+                [self.options.docker_binary, 'inspect', '--format={{.State.Running}}', cid],
                 env=os.environ.copy(),
                 close_fds=True,
                 stdout=subprocess.PIPE
             )
             scode_call.wait()
             parsed_scode = scode_call.stdout.read().strip()
+            if six.PY3:
+                parsed_scode = parsed_scode.decode(__salt_system_encoding__)
             if parsed_scode != 'false':
                 # If the container is still running, let's make sure it
                 # properly stops
-                print(' * Making sure the container is stopped. CID:'),
+                sys.stdout.write(' * Making sure the container is stopped. CID: ')
                 sys.stdout.flush()
 
                 stop_call = subprocess.Popen(
-                    ['docker', 'stop', '--time=15', cid],
+                    [self.options.docker_binary, 'stop', '--time=15', cid],
                     env=os.environ.copy(),
                     close_fds=True,
                     stdout=subprocess.PIPE
                 )
                 stop_call.wait()
-                print(stop_call.stdout.read().strip())
+                output = stop_call.stdout.read().strip()
+                if six.PY3:
+                    output = output.decode(__salt_system_encoding__)
+                print(output)
                 sys.stdout.flush()
                 time.sleep(0.5)
 
             # Let's get the container's exit code. We can't trust on Popen's
             # returncode because it's not reporting the proper one? Still
             # haven't narrowed it down why.
-            print(' * Container exit code:'),
+            sys.stdout.write(' * Container exit code: ')
             sys.stdout.flush()
             rcode_call = subprocess.Popen(
-                ['docker', 'inspect', '-format={{.State.ExitCode}}', cid],
+                [self.options.docker_binary, 'inspect', '--format={{.State.ExitCode}}', cid],
                 env=os.environ.copy(),
                 close_fds=True,
                 stdout=subprocess.PIPE
             )
             rcode_call.wait()
             parsed_rcode = rcode_call.stdout.read().strip()
+            if six.PY3:
+                parsed_rcode = parsed_rcode.decode(__salt_system_encoding__)
             try:
                 returncode = int(parsed_rcode)
             except ValueError:
@@ -692,18 +732,21 @@ class SaltTestingParser(optparse.OptionParser):
 
             if self.options.docked_skip_delete is False and \
                     (self.options.docked_skip_delete_on_errors is False or
-                    (self.options.docked_skip_delete_on_error and
-                     returncode == 0)):
-                print(' * Cleaning Up Temporary Docker Container. CID:'),
+                            (self.options.docked_skip_delete_on_error and
+                                returncode == 0)):
+                sys.stdout.write(' * Cleaning Up Temporary Docker Container. CID: ')
                 sys.stdout.flush()
                 cleanup_call = subprocess.Popen(
-                    ['docker', 'rm', cid],
+                    [self.options.docker_binary, 'rm', cid],
                     env=os.environ.copy(),
                     close_fds=True,
                     stdout=subprocess.PIPE
                 )
                 cleanup_call.wait()
-                print(cleanup_call.stdout.read().strip())
+                output = cleanup_call.stdout.read().strip()
+                if six.PY3:
+                    output = output.decode(__salt_system_encoding__)
+                print(output)
 
             if 'DOCKER_CIDFILE' not in os.environ:
                 # The CID file was not created "from the outside", so delete it
@@ -745,13 +788,9 @@ class SaltTestingParser(optparse.OptionParser):
 
             elif option.action == 'append':
                 for val in (value is not None and value or default):
-                    calling_args.extend(
-                       [option.get_opt_string(), str(val)]
-                )
+                    calling_args.extend([option.get_opt_string(), str(val)])
             elif option.action == 'count':
-                calling_args.extend(
-                    [option.get_opt_string()] * value
-                )
+                calling_args.extend([option.get_opt_string()] * value)
             else:
                 calling_args.extend(
                     [option.get_opt_string(),
@@ -766,9 +805,9 @@ class SaltTestingParser(optparse.OptionParser):
                 '-{0}'.format('v' * (self.options.verbosity - 1))
             )
 
-        print(' * Docker command: {0}'.format(' '.join(calling_args)))
-        print(' * Running the tests suite under the {0!r} docker '
-              'container. CID:'.format(container)),
+        sys.stdout.write(' * Docker command: {0}\n'.format(' '.join(calling_args)))
+        sys.stdout.write(' * Running the tests suite under the {0!r} docker '
+                         'container. CID: '.format(container))
         sys.stdout.flush()
 
         cidfile = os.environ.get(
@@ -792,7 +831,7 @@ class SaltTestingParser(optparse.OptionParser):
              'COLUMNS={0}'.format(WIDTH),
              '-e',
              'LINES={0}'.format(HEIGHT),
-             '-cidfile={0}'.format(cidfile),
+             '--cidfile={0}'.format(cidfile),
              container,
              # We need to pass the runtests.py arguments as a single string so
              # that the start-me-up.sh script can handle them properly
