@@ -601,8 +601,12 @@ def bootstrap_parallels_minion(options):
         '''
         # TODO: This function needs to be refactored into bootstrap for MacOS
 
-        pkg_name = 'salt-{0}-x86_64.pkg'.format(options.bootstrap_salt_commit.lstrip('v'))
-
+        # Fixing MAC Packages to have py2 and py3 in the package names.
+        mac_py = 'py2'
+        if options.test_with_python3:
+            mac_py = 'py3'
+        pkg_name = 'salt-{0}-{1}-x86_64.pkg'.format(options.bootstrap_salt_commit.lstrip('v'), mac_py)
+        print_bulleted(options, 'Package Name is: ' + pkg_name, 'LIGHT_GREEN')
         # Wait for network to finish configuring (and VM to finish starting)
         ping_cmd = 'ping -c 3 repo.saltstack.com'
         ping_wrap = _prl_cmd('exec', options.vm_name, command=pipes.quote(ping_cmd))
@@ -621,7 +625,10 @@ def bootstrap_parallels_minion(options):
         downl_wrap = _prl_cmd('exec', options.vm_name, command=pipes.quote(downl_cmd))
         run_command(downl_wrap, options)
         # Wait for package to finish downloading by first downloading the hash
-        hash_cmd = 'cat /tmp/{0}.md5'.format(pkg_name)
+
+        # Stripped out the package name from the .md5 file for the comparison
+        hash_cut_cmd = '; echo $1 | perl -ne \\"print lc\\"'
+        hash_cmd = 'set `cat /tmp/{0}.md5` {1}'.format(pkg_name, hash_cut_cmd)
         hash_wrap = _prl_cmd('exec', options.vm_name, command=pipes.quote(hash_cmd))
         hash_code = run_command(hash_wrap, options, return_output=True)[0]
         # And then comparing it to the actual hash
@@ -725,7 +732,12 @@ def prepare_ssh_access(options):
         roster_fh.close()
 
         # We also need a master config file. Ours will point to the salt-jenkins repo and mount it with GitFS
-        ssh_master_conf_data = {'fileserver_backend': ['git'], 'gitfs_remotes': ['https://github.com/saltstack/salt-jenkins.git']}
+        ssh_master_conf_data = {
+            'fileserver_backend': ['git'],
+            'gitfs_remotes': [
+                '{} {}'.format(options.salt_jenkins_repo, options.salt_jenkins_branch),
+            ],
+        }
 
         with open('/tmp/.jenkins_ssh/master', 'w') as master_fh:
             yaml.dump(ssh_master_conf_data, stream=master_fh)
@@ -936,9 +948,8 @@ def get_minion_python_executable(options):
             python_executable = 'C:\\PROGRA~1\\Python35\\python.exe'
         else:
             python_executable = 'C:\\Python27\\python.exe'
-    elif options.parallels_deploy:
-        if options.test_with_python3:
-            python_executable = '/usr/local/bin/python3'
+    elif options.parallels_deploy and options.test_with_python3:
+        python_executable = '/opt/salt/bin/python3'
     else:
         if options.test_with_python3:
             python_executable = '/usr/bin/python3'
@@ -2131,6 +2142,16 @@ def get_args():
         action='store_true',
         help='Print out the default command that runs the test suite on the deployed VM'
     )
+    testing_source_options.add_argument(
+        '--salt-jenkins-repo',
+        default=os.environ.get('SALT_JENKINS_REPO', 'https://github.com/saltstack/salt-jenkins.git'),
+        help='The repository that contains the environment prep states, usually salt-jenkins.'
+    )
+    testing_source_options.add_argument(
+        '--salt-jenkins-branch',
+        default=os.environ.get('SALT_JENKINS_BRANCH', 'develop'),
+        help='The branch of the `salt-jenkins-repo` to use..'
+    )
     testing_source_options.add_argument(
         '--update-winrepo',
         action='store_true',
